@@ -1,17 +1,19 @@
 # ABSTRACT: create new Dancer2 application
 package Dancer2::CLI::Command::gen;
-$Dancer2::CLI::Command::gen::VERSION = '0.152000';
+$Dancer2::CLI::Command::gen::VERSION = '0.160003';
 use strict;
 use warnings;
 
+use App::Cmd::Setup -command;
+
+use HTTP::Tiny;
 use File::Find;
-use LWP::UserAgent;
 use File::Path 'mkpath';
 use File::Spec::Functions;
 use File::ShareDir 'dist_dir';
 use File::Basename qw/dirname basename/;
 use Dancer2::Template::Simple;
-use App::Cmd::Setup -command;
+use Class::Load 'try_load_class';
 
 my $SKEL_APP_FILE = 'lib/AppFile.pm';
 
@@ -84,7 +86,7 @@ sub execute {
     _create_manifest($files_to_copy, $app_path);
     _add_to_manifest_skip($app_path);
 
-    unless (eval 'require YAML') {
+    if ( ! try_load_class('YAML') ) {
         print <<NOYAML;
 *****
 WARNING: YAML.pm is not installed.  This is not a full dependency, but is highly
@@ -106,7 +108,7 @@ NOYAML
 
 sub version {
     require Dancer2;
-    return $Dancer2::VERSION;
+    return Dancer2->VERSION;
 }
 
 # skel creation routines
@@ -257,12 +259,10 @@ Please check http://search.cpan.org/dist/Dancer2/ for updates.
 sub _send_http_request {
     my $url = shift;
 
-    my $ua = LWP::UserAgent->new;
-    $ua->timeout(5);
-    $ua->env_proxy();
+    my $ua = HTTP::Tiny->new( timeout => 5 );
 
     my $response = $ua->get($url);
-    return $response->is_success ? $response->content : undef;
+    return $response->{'success'} ? $response->{'content'} : undef;
 }
 
 1;
@@ -279,7 +279,7 @@ Dancer2::CLI::Command::gen - create new Dancer2 application
 
 =head1 VERSION
 
-version 0.152000
+version 0.160003
 
 =head1 AUTHOR
 
@@ -287,7 +287,7 @@ Dancer Core Developers
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2014 by Alexis Sukrieh.
+This software is copyright (c) 2015 by Alexis Sukrieh.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
