@@ -1,12 +1,13 @@
 package Dancer2::Core::Role::Hookable;
 # ABSTRACT: Role for hookable objects
-$Dancer2::Core::Role::Hookable::VERSION = '0.152000';
+$Dancer2::Core::Role::Hookable::VERSION = '0.160003';
 use Moo::Role;
 use Dancer2::Core;
 use Dancer2::Core::Types;
 use Carp 'croak';
+use Safe::Isa;
 
-requires 'supported_hooks';
+requires 'supported_hooks', 'hook_aliases';
 
 # The hooks registry
 has hooks => (
@@ -18,34 +19,6 @@ has hooks => (
 
 sub BUILD { }
 
-# this hash contains all known core hooks with their 'human' name
-# classes that consume the role can override this method to provide
-# their own aliases for their own hooks
-sub hook_aliases {
-    {   before                 => 'core.app.before_request',
-        before_request         => 'core.app.before_request',
-        after                  => 'core.app.after_request',
-        after_request          => 'core.app.after_request',
-        before_file_render     => 'handler.file.before_render',
-        after_file_render      => 'handler.file.after_render',
-        before_template_render => 'engine.template.before_render',
-        after_template_render  => 'engine.template.after_render',
-        before_layout_render   => 'engine.template.before_layout_render',
-        after_layout_render    => 'engine.template.after_layout_render',
-        before_serializer      => 'engine.serializer.before',
-        after_serializer       => 'engine.serializer.after',
-        init_error             => 'core.error.init',
-        before_error           => 'core.error.before',
-        after_error            => 'core.error.after',
-        on_route_exception     => 'core.app.route_exception',
-
-        # compatibility from Dancer1
-        before_error_render    => 'core.error.before',
-        after_error_render     => 'core.error.after',
-        before_error_init      => 'core.error.init',
-    };
-}
-
 # after a hookable object is built, we go over its postponed hooks and register
 # them if any.
 after BUILD => sub {
@@ -61,7 +34,7 @@ sub _add_postponed_hooks {
     # find the internal name of the hooks, from the caller name
     my $caller = ref($self);
     my ( $dancer, $h_type, $h_name, @rest ) = map {lc} split /::/, $caller;
-    $h_name = $rest[0] if $h_name eq 'Role';
+    $h_name = $rest[0] if $h_name eq 'role';
     if ( $h_type =~ /(template|logger|serializer|session)/ ) {
         $h_name = $h_type;
         $h_type = 'engine';
@@ -134,8 +107,8 @@ sub execute_hook {
     croak "Hook '$name' does not exist"
       if !$self->has_hook($name);
 
-    ref($self) eq 'Dancer2::Core::App' &&
-        $self->engine('logger')->core("Entering hook $name");
+    $self->$_isa('Dancer2::Core::App') &&
+      $self->log( core => "Entering hook $name" );
 
     for my $hook ( @{ $self->hooks->{$name} } ) {
         $hook->(@args);
@@ -156,7 +129,7 @@ Dancer2::Core::Role::Hookable - Role for hookable objects
 
 =head1 VERSION
 
-version 0.152000
+version 0.160003
 
 =head1 AUTHOR
 
@@ -164,7 +137,7 @@ Dancer Core Developers
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2014 by Alexis Sukrieh.
+This software is copyright (c) 2015 by Alexis Sukrieh.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
