@@ -1,14 +1,18 @@
 package Dancer2::Plugin;
 # ABSTRACT: Extending Dancer2's DSL with plugins
-$Dancer2::Plugin::VERSION = '0.152000';
+$Dancer2::Plugin::VERSION = '0.160003';
 use Moo::Role;
 use Carp 'croak', 'carp';
 use Dancer2::Core::DSL;
+use Scalar::Util qw();
 
 # singleton for storing all keywords,
 # their code and the plugin they come from
 my $_keywords = {};
 
+# singleton for storing all hooks and their aliases
+my $_hooks = {};
+
 # singleton for applying code-blocks at import time
 # so their code gets the callers DSL
 my $_on_import = {};
@@ -21,7 +25,7 @@ sub register {
 
     $keyword =~ /^[a-zA-Z_]+[a-zA-Z0-9_]*$/
       or croak "You can't use '$keyword', it is an invalid name"
-      . " (it should match ^[a-zA-Z_]+[a-zA-Z0-9_]*$ )";
+      . " (it should match ^[a-zA-Z_]+[a-zA-Z0-9_]*\$ )";
 
     if (grep { $_ eq $keyword }
         keys %{ Dancer2::Core::DSL->dsl_keywords }
@@ -39,7 +43,7 @@ sub register {
 
     $_keywords->{$plugin} ||= [];
     push @{ $_keywords->{$plugin} },
-      [ $keyword, $code, $options->{is_global} ];
+      [ $keyword, $code, $options ];
 }
 
 sub on_plugin_import(&) {
@@ -63,10 +67,10 @@ sub register_plugin {
     # bind all registered keywords to the plugin
     my $dsl = $caller->dsl;
     for my $k ( @{ $_keywords->{$plugin} } ) {
-        my ( $keyword, $code, $is_global ) = @{$k};
+        my ( $keyword, $code, $options ) = @{$k};
         {
             no strict 'refs';
-            *{"${plugin}::${keyword}"} = $code;
+            *{"${plugin}::${keyword}"} = $dsl->_apply_prototype($code, $options);
         }
     }
 
@@ -79,7 +83,8 @@ sub register_plugin {
         my $caller = caller(1);
 
         for my $k ( @{ $_keywords->{$plugin} } ) {
-            my ( $keyword, $code, $is_global ) = @{$k};
+            my ( $keyword, $code, $options ) = @{$k};
+            my $is_global = exists $options->{is_global} && $options->{is_global};
             $caller->dsl->register( $keyword, $is_global );
         }
 
@@ -87,6 +92,25 @@ sub register_plugin {
         $caller->dsl->export_symbols_to($caller);
         $caller->dsl->dancer_app->register_plugin( $caller->dsl );
 
+        # add hooks
+        my $current_hooks = [ $caller->dsl->supported_hooks ];
+        my $current_aliases = $caller->dsl->hook_aliases;
+        for my $h ( keys %{ $_hooks->{$plugin} } ) {
+            push @$current_hooks, $h;
+            $current_aliases->{ $_hooks->{$plugin}->{$h} } = $h;
+			# If the hooks atttribute has already been constructed,
+			# add an entry so has_hook() finds these hooks.
+            $caller->dsl->hooks->{$h} = []
+                if ! exists $caller->dsl->hooks->{$h};
+        }
+        my $target = ref $caller->dsl;
+        {
+            no strict 'refs';
+            no warnings 'redefine';
+            *{"${target}::supported_hooks"} = sub {@$current_hooks};
+            *{"${target}::hook_aliases"}    = sub {$current_aliases};
+        }
+
         for my $sub ( @{ $_on_import->{$plugin} } ) {
             $sub->( $caller->dsl );
         }
@@ -111,27 +135,19 @@ sub plugin_args {@_}
 
 sub plugin_setting {
     my $plugin = caller;
-    my $dsl    = _get_dsl();
+    my $dsl    = _get_dsl()
+        or croak 'No DSL object found';
+
     ( my $plugin_name = $plugin ) =~ s/Dancer2::Plugin:://;
-    my $app = $dsl->dancer_app;
-    return $app->config->{'plugins'}->{$plugin_name} ||= {};
+
+    return $dsl->app->config->{'plugins'}->{$plugin_name} ||= {};
 }
 
 sub register_hook {
-    my $caller = caller;
-    my $plugin = $caller;
-
     my (@hooks) = @_;
 
-    my $current_hooks = [];
-    if ( $plugin->can('supported_hooks') ) {
-        $current_hooks = [ $plugin->supported_hooks ];
-    }
-
-    my $current_aliases = {};
-    if ( $plugin->can('hook_aliases') ) {
-        $current_aliases = $plugin->hook_aliases;
-    }
+    my $caller = caller;
+    my $plugin = $caller;
 
     $plugin =~ s/^Dancer2::Plugin:://;
     $plugin =~ s/::/_/g;
@@ -139,16 +155,7 @@ sub register_hook {
     my $base_name = "plugin." . lc($plugin);
     for my $hook (@hooks) {
         my $hook_name = "${base_name}.$hook";
-
-        push @{$current_hooks}, $hook_name;
-        $current_aliases->{$hook} = $hook_name;
-    }
-
-    {
-        no strict 'refs';
-        no warnings 'redefine';
-        *{"${caller}::supported_hooks"} = sub {@$current_hooks};
-        *{"${caller}::hook_aliases"}    = sub {$current_aliases};
+        $_hooks->{$caller}->{$hook_name} = $hook;
     }
 }
 
@@ -211,8 +218,12 @@ sub import {
                 $code->( $dsl, @_ );
             };
 
-            # bind the newly compiled symbol to the caller's namespace.
-            *{"${plugin}::${symbol}"} = $compiled;
+            # Bind the newly compiled symbol to the caller's namespace.
+            # As this may redefine a symbol, ensure the new coderef has
+            # the same prototype signature.
+            my $existing = *{"${plugin}::${symbol}"};
+            my $prototype = prototype \&$existing;
+            *{"${plugin}::${symbol}"} = Scalar::Util::set_prototype( \&$compiled, $prototype );
 
             $dsl_deprecation_wrapper = $compiled if $symbol eq 'dsl';
         }
@@ -251,7 +262,7 @@ Dancer2::Plugin - Extending Dancer2's DSL with plugins
 
 =head1 VERSION
 
-version 0.152000
+version 0.160003
 
 =head1 DESCRIPTION
 
@@ -439,7 +450,7 @@ Dancer Core Developers
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2014 by Alexis Sukrieh.
+This software is copyright (c) 2015 by Alexis Sukrieh.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
